@@ -1,4 +1,3 @@
-# $Id: Sub.pm 42 2008-06-30 08:47:59Z dave $
 
 =head1 NAME
 
@@ -12,17 +11,17 @@ Symbol::Approx::Sub - Perl module for calling subroutines by approximate names!
     # blah...
   }
 
-  &aa; # executes &a if &aa doesn't exist.
+  aa(); # executes a() if aa() doesn't exist.
 
   use Symbol::Approx::Sub (xform => 'Text::Metaphone');
   use Symbol::Approx::Sub (xform => undef,
-			   match => 'String::Approx');
+                           match => 'String::Approx');
   use Symbol::Approx::Sub (xform => 'Text::Soundex');
   use Symbol::Approx::Sub (xform => \&my_transform);
   use Symbol::Approx::Sub (xform => [\&my_transform, 'Text::Soundex']);
   use Symbol::Approx::Sub (xform => \&my_transform,
-			   match => \&my_matcher,
-			   choose => \&my_chooser);
+                           match => \&my_matcher,
+                           choose => \&my_chooser);
 
 
 =head1 DESCRIPTION
@@ -63,7 +62,7 @@ There are three phases to the matching process. They are:
 
 B<transform> - a transform subroutine applies some kind of transformation
 to the subroutine names. For example the default transformer applies the
-Soundex algorithm to each of the subroutine names. Other obvious 
+Soundex algorithm to each of the subroutine names. Other obvious
 tranformations would be to remove all the underscores or to change the
 names to lower case.
 
@@ -87,7 +86,7 @@ default matcher simply checks to see if the strings are equal.
 
 A match subroutine is passed the target string as its first parameter,
 followed by the list of potential matches. For each string that matches,
-the matcher should return the index number from the input list. For example, 
+the matcher should return the index number from the input list. For example,
 the default matcher is implemented like this:
 
   sub matcher {
@@ -107,7 +106,7 @@ B<choose> - a chooser subroutine takes a list of matches and chooses exactly
 one item from the list. The default matcher chooses one item at random.
 
 A chooser subroutine is passed a list of matches and must simply return one
-index number from that list. For example, the default chooser is implemented 
+index number from that list. For example, the default chooser is implemented
 like this:
 
   sub chooser {
@@ -118,7 +117,7 @@ like this:
 
 You can override any of these behaviours by writing your own transformer,
 matcher or chooser. You can either define the subroutine in your own
-script or you can put the subroutine in a separate module which 
+script or you can put the subroutine in a separate module which
 Symbol::Approx::Sub can then use as a I<plug-in>. See below for more details
 on plug-ins.
 
@@ -154,20 +153,23 @@ use warnings;
 our ($VERSION, @ISA, $AUTOLOAD);
 
 use Devel::Symdump;
+use Module::Load;
+use Exception::Class (
+  'SAS::Exception',
+  'SAS::Exception::InvalidOption'              => { isa => 'SAS::Exception' },
+  'SAS::Exception::InvalidOption::Transformer' => { isa => 'SAS::Exception::InvalidOption' },
+  'SAS::Exception::InvalidOption::Matcher'     => { isa => 'SAS::Exception::InvalidOption' },
+  'SAS::Exception::InvalidOption::Chooser'     => { isa => 'SAS::Exception::InvalidOption' },
+  'SAS::Exception::InvalidParameter'           => { isa => 'SAS::Exception' },
+);
 
-$VERSION = sprintf "%d", '$Revision: 42 $ ' =~ /(\d+)/;
+$VERSION = '3.0.0';
 
 use Carp;
 
 # List of functions that we _never_ try to match approximately.
 my @_BARRED = qw(AUTOLOAD BEGIN CHECK INIT DESTROY END);
-my %_BARRED = (1) x @_BARRED;
-
-sub _pkg2file {
-  $_ = shift;
-  s|::|/|g;
-  "$_.pm";
-}
+my %_BARRED = map { $_ => 1 } @_BARRED;
 
 # import is called when another script uses this module.
 # All we do here is overwrite the caller's AUTOLOAD subroutine
@@ -191,113 +193,21 @@ sub import  {
   my %CONF;
   %param = @_ if @_;
 
-  my %defaults = (xform => 'Text::Soundex',
-		  match => 'String::Equal',
-		  choose => 'Random');
+  my %defaults = (
+    xform  => 'Text::Soundex',
+    match  => 'String::Equal',
+    choose => 'Random'
+  );
 
-  # Work out which transformer(s) to use. The valid options are:
-  # 1/ $param{xform} doesn't exist. Use default transformer.
-  # 2/ $param{xform} is undef. Use no transformers.
-  # 3/ $param{xform} is a reference to a subroutine. Use the 
-  #    referenced subroutine as the transformer.
-  # 4/ $param{xform} is a scalar. This is the name of a transformer
-  #    module which should be loaded.
-  # 5/ $param{xform} is a reference to an array. Each element of the
-  #    array is one of the previous two options.
-
-  if (exists $param{xform}) {
-    if (defined $param{xform}) {
-      my $type = ref $param{xform};
-      if ($type eq 'CODE') {
-	$CONF{xform} = [$param{xform}];
-      } elsif ($type eq '') {
-	my $mod = "Symbol::Approx::Sub::$param{xform}";
-	require(_pkg2file($mod));
-	$CONF{xform} = [\&{"${mod}::transform"}];
-      } elsif ($type eq 'ARRAY') {
-	foreach (@{$param{xform}}) {
-	  my $type = ref $_;
-	  if ($type eq 'CODE') {
-	    push @{$CONF{xform}}, $_;
-	  } elsif ($type eq '') {
-	    my $mod = "Symbol::Approx::Sub::$_";
-	    require(_pkg2file($mod));
-	    push @{$CONF{xform}}, \&{"${mod}::transform"};
-	  } else {
-	    croak 'Invalid transformer passed to Symbol::Approx::Sub';
-	  }
-	}
-      } else {
-	croak 'Invalid transformer passed to Symbol::Approx::Sub';
-      }
-    } else {
-      $CONF{xform} = [];
-    }
-  } else {
-    my $mod = "Symbol::Approx::Sub::$defaults{xform}";
-    require(_pkg2file($mod));
-    $CONF{xform} = [\&{"${mod}::transform"}];
+  foreach (keys %param) {
+    SAS::Exception::InvalidParameter->throw(
+      error => "Invalid parameter $_\n",
+    ) unless exists $defaults{$_};
   }
 
-  # Work out which matcher to use. The valid options are:
-  # 1/ $param{match} doesn't exist. Use default matcher.
-  # 2/ $param{match} is undef. Use no matcher.
-  # 3/ $param{match} is a reference to a subroutine. Use the 
-  #    referenced subroutine as the matcher.
-  # 4/ $param{match} is a scalar. This is the name of a matcher
-  #    module which should be loaded.
-
-  if (exists $param{match}) {
-    if (defined $param{match}) {
-      my $type = ref $param{match};
-      if ($type eq 'CODE') {
-	$CONF{match} = $param{match};
-      } elsif ($type eq '') {
-	my $mod = "Symbol::Approx::Sub::$param{match}";
-	require(_pkg2file($mod));
-	$CONF{match} = \&{"${mod}::match"};
-      } else {
-	croak 'Invalid matcher passed to Symbol::Approx::Sub';
-      }
-    } else {
-      $CONF{match} = undef;
-    }
-  } else {
-    my $mod = "Symbol::Approx::Sub::$defaults{match}";
-    require(_pkg2file($mod));
-    $CONF{match} = \&{"${mod}::match"};
-  }
-
-  # Work out which chooser to use. The valid options are:
-  # 1/ $param{choose} doesn't exist. Use default chooser.
-  # 2/ $param{choose} is undef. Use default chooser.
-  # 3/ $param{choose} is a reference to a subroutine. Use the 
-  #    referenced subroutine as the chooser.
-  # 4/ $param{choose} is a scalar. This is the name of a chooser
-  #    module which should be loaded.
-
-  if (exists $param{choose}) {
-    if (defined $param{choose}) {
-      my $type = ref $param{choose};
-      if ($type eq 'CODE') {
-	$CONF{chooser} = $param{chooser};
-      } elsif ($type eq '') {
-	my $mod = "Symbol::Approx::Sub::$param{choose}";
-	require(_pkg2file($mod));
-	$CONF{choose} = \&{"${mod}::choose"};
-      } else {
-	croak 'Invalid chooser passed to Symbol::Approx::Sub';
-      }
-    } else {
-      my $mod = "Symbol::Approx::Sub::$defaults{choose}";
-      require(_pkg2file($mod));
-      $CONF{choose} = \&{"4mod::choose"};
-    }
-  } else {
-    my $mod = "Symbol::Approx::Sub::$defaults{choose}";
-    require(_pkg2file($mod));
-    $CONF{choose} = \&{"${mod}::choose"};
-  }
+  _set_transformer(\%param, \%CONF, $defaults{xform});
+  _set_matcher(\%param, \%CONF, $defaults{match});
+  _set_chooser(\%param, \%CONF, $defaults{choose});
 
   # Now install appropriate AUTOLOAD routine in caller's package
 
@@ -305,6 +215,133 @@ sub import  {
   *{"${pkg}::AUTOLOAD"} = _make_AUTOLOAD(%CONF);
 }
 
+# Work out which transformer(s) to use. The valid options are:
+# 1/ $param{xform} doesn't exist. Use default transformer.
+# 2/ $param{xform} is undef. Use no transformers.
+# 3/ $param{xform} is a reference to a subroutine. Use the
+#    referenced subroutine as the transformer.
+# 4/ $param{xform} is a scalar. This is the name of a transformer
+#    module which should be loaded.
+# 5/ $param{xform} is a reference to an array. Each element of the
+#    array is one of the previous two options.
+sub _set_transformer {
+  my ($param, $CONF, $default) = @_;
+
+  unless (exists $param->{xform}) {
+    my $mod = "Symbol::Approx::Sub::$default";
+    load $mod;
+    $CONF->{xform} = [\&{"${mod}::transform"}];
+    return;
+  }
+
+  unless (defined $param->{xform}) {
+    $CONF->{xform} = [];
+    return;
+  }
+
+  my $type = ref $param->{xform};
+  if ($type eq 'CODE') {
+    $CONF->{xform} = [$param->{xform}];
+  } elsif ($type eq '') {
+    my $mod = "Symbol::Approx::Sub::$param->{xform}";
+    load $mod;
+    $CONF->{xform} = [\&{"${mod}::transform"}];
+  } elsif ($type eq 'ARRAY') {
+    foreach (@{$param->{xform}}) {
+      my $type = ref $_;
+      if ($type eq 'CODE') {
+        push @{$CONF->{xform}}, $_;
+      } elsif ($type eq '') {
+        my $mod = "Symbol::Approx::Sub::$_";
+        load $mod;
+        push @{$CONF->{xform}}, \&{"${mod}::transform"};
+      } else {
+        SAS::Exception::InvalidOption::Transformer->throw(
+          error => 'Invalid transformer passed to Symbol::Approx::Sub'
+        );
+      }
+    }
+  } else {
+    SAS::Exception::InvalidOption::Transformer->throw(
+      error => 'Invalid transformer passed to Symbol::Approx::Sub'
+    );
+  }
+}
+
+# Work out which matcher to use. The valid options are:
+# 1/ $param{match} doesn't exist. Use default matcher.
+# 2/ $param{match} is undef. Use no matcher.
+# 3/ $param{match} is a reference to a subroutine. Use the
+#    referenced subroutine as the matcher.
+# 4/ $param{match} is a scalar. This is the name of a matcher
+#    module which should be loaded.
+sub _set_matcher {
+  my ($param, $CONF, $default) = @_;
+
+  unless (exists $param->{match}) {
+    my $mod = "Symbol::Approx::Sub::$default";
+    load $mod;
+    $CONF->{match} = \&{"${mod}::match"};
+    return;
+  }
+
+  unless (defined $param->{match}) {
+    $CONF->{match} = undef;
+    return;
+  }
+
+  my $type = ref $param->{match};
+  if ($type eq 'CODE') {
+    $CONF->{match} = $param->{match};
+  } elsif ($type eq '') {
+    my $mod = "Symbol::Approx::Sub::$param->{match}";
+    load $mod;
+    $CONF->{match} = \&{"${mod}::match"};
+  } else {
+    SAS::Exception::InvalidOption::Matcher->throw(
+      error => 'Invalid matcher passed to Symbol::Approx::Sub'
+    );
+  }
+}
+
+# Work out which chooser to use. The valid options are:
+# 1/ $param{choose} doesn't exist. Use default chooser.
+# 2/ $param{choose} is undef. Use default chooser.
+# 3/ $param{choose} is a reference to a subroutine. Use the
+#    referenced subroutine as the chooser.
+# 4/ $param{choose} is a scalar. This is the name of a chooser
+#    module which should be loaded.
+sub _set_chooser {
+  my ($param, $CONF, $default) = @_;
+
+  unless (exists $param->{choose}) {
+    my $mod = "Symbol::Approx::Sub::$default";
+    load $mod;
+    $CONF->{choose} = \&{"${mod}::choose"};
+    return;
+  }
+
+  unless (defined $param->{choose}) {
+    my $mod = "Symbol::Approx::Sub::$default";
+    load $mod;
+    $CONF->{choose} = \&{"${mod}::choose"};
+    return;
+  }
+
+  my $type = ref $param->{choose};
+  if ($type eq 'CODE') {
+    $CONF->{choose} = $param->{choose};
+  } elsif ($type eq '') {
+    my $mod = "Symbol::Approx::Sub::$param->{choose}";
+    load $mod;
+    $CONF->{choose} = \&{"${mod}::choose"};
+  } else {
+    SAS::Exception::InvalidOption::Chooser->throw(
+      error => 'Invalid chooser passed to Symbol::Approx::Sub',
+    );
+  }
+}
+
 # Create a subroutine which is called when a given subroutine
 # name can't be found in the current package. In the import subroutine
 # above, we have already arranged that our calling package will use
@@ -322,32 +359,30 @@ sub _make_AUTOLOAD {
     # in the %_BARRED hash
     my (@subs, @orig);
     my $sym = Devel::Symdump->new($pkg);
-    @orig = @subs = grep { ! $_BARRED{$_} } 
+    @orig = @subs = grep { ! $_BARRED{$_} }
                     map { s/${pkg}:://; $_ }
-                    grep { defined &{$_} } $sym->functions($pkg);
-
-    unshift @subs, $sub;
+                    grep { defined &{$_} } $sym->functions();
 
     # Transform all of the subroutine names
     foreach (@{$CONF{xform}}) {
-      croak "Invalid transformer passed to Symbol::Approx::Sub\n"
-	unless defined &$_;
-      @subs = $_->(@subs);
+      SAS::Exception::InvalidOption::Transformer->throw(
+        error => 'Invalid transformer passed to Symbol::Approx::Sub',
+      ) unless defined &$_;
+      ($sub, @subs) = $_->($sub, @subs);
     }
 
     # Call the subroutine that will look for matches
     # The matcher returns a list of the _indexes_ that match
     my @match_ind;
     if ($CONF{match}) {
-      croak "Invalid matcher passed to Symbol::Approx::Sub\n"
-	unless defined &{$CONF{match}};
-      @match_ind = $CONF{match}->(@subs);
+      SAS::Exception::InvalidOption::Matcher->throw(
+        error => 'Invalid matcher passed to Symbol::Approx::Sub',
+      ) unless defined &{$CONF{match}};
+      @match_ind = $CONF{match}->($sub, @subs);
     } else {
-      @match_ind = @subs[1 .. $#subs];
+      @match_ind = (0 .. $#subs);
     }
 
-    shift @subs;
-
     @subs = @subs[@match_ind];
     @orig = @orig[@match_ind];
 
@@ -359,8 +394,9 @@ sub _make_AUTOLOAD {
       if (@match_ind == 1) {
         $sub = "${pkg}::" . $orig[0];
       } else {
-	croak "Invalid chooser passed to Symbol::Approx::Sub\n"
-	  unless defined $CONF{choose};
+        SAS::Exception::InvalidOption::Chooser->throw(
+          error => 'Invalid chooser passed to Symbol::Approx::Sub'
+        ) unless defined $CONF{choose};
         $sub = "${pkg}::" . $orig[$CONF{choose}->(@subs)];
       }
       goto &$sub;
@@ -375,8 +411,8 @@ __END__
 
 =head1 CAVEAT
 
-I can't stress too strongly that this will make your code completely 
-unmaintainable and you really shouldn't use this module unless you're 
+I can't stress too strongly that this will make your code completely
+unmaintainable and you really shouldn't use this module unless you're
 doing something very stupid.
 
 =head1 ACKNOWLEDGEMENTS
@@ -406,7 +442,7 @@ With lots of help from Leon Brocard <leon@astray.com>
 Copyright (C) 2000-2008, Magnum Solutions Ltd.  All Rights Reserved.
 
 This script is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself. 
+under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
